use std::collections::HashMap;
use std::sync::{OnceLock, Mutex};
use anyhow::Result;
use serde::{Deserialize, Serialize};
use std::fs;
use std::path::Path;

#[derive(Debug, Serialize, Deserialize)]
pub struct Config {
    settings: HashMap<String, String>,
}

impl Config {
    // 获取配置实例
    pub fn get_instance() -> &'static Mutex<Config> {
        static INSTANCE: OnceLock<Mutex<Config>> = OnceLock::new();
        INSTANCE.get_or_init(|| Mutex::new(Config::default()))
    }

    // 从文件加载配置
    pub fn load_from_file<P: AsRef<Path>>(path: P) -> Result<()> {
        let config_str = fs::read_to_string(path)?;
        let new_config: Config = serde_json::from_str(&config_str)?;
        
        let instance = Self::get_instance().lock().unwrap();
        let mut instance = instance;
        *instance = new_config;
        
        Ok(())
    }

    // 获取配置项
    pub fn get_setting(&self, key: &str) -> Option<&String> {
        self.settings.get(key)
    }

    // 设置配置项
    pub fn set_setting(&mut self, key: String, value: String) {
        self.settings.insert(key, value);
    }

    // 保存配置到文件
    pub fn save_to_file<P: AsRef<Path>>(&self, path: P) -> Result<()> {
        let config_str = serde_json::to_string_pretty(self)?;
        fs::write(path, config_str)?;
        Ok(())
    }
}

// 默认配置
impl Default for Config {
    fn default() -> Self {
        let mut settings = HashMap::new();
        settings.insert("case_sensitive".to_string(), "true".to_string());
        settings.insert("max_word_length".to_string(), "100".to_string());
        settings.insert("min_word_length".to_string(), "1".to_string());
        
        Config { settings }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use std::fs;
    use std::io::Write;
    use tempfile::NamedTempFile;

    #[test]
    fn test_singleton_pattern() {
        let instance1 = Config::get_instance();
        let instance2 = Config::get_instance();
        
        // 验证两个实例是同一个
        assert_eq!(
            format!("{:p}", instance1),
            format!("{:p}", instance2)
        );
    }

    #[test]
    fn test_config_operations() {
        let instance = Config::get_instance();
        let mut config = instance.lock().unwrap();
        
        // 测试设置和获取配置
        config.set_setting("test_key".to_string(), "test_value".to_string());
        assert_eq!(
            config.get_setting("test_key"),
            Some(&"test_value".to_string())
        );
    }

    #[test]
    fn test_file_operations() -> Result<()> {
        // 创建临时文件
        let mut temp_file = NamedTempFile::new()?;
        let test_config = r#"{"settings":{"test_key":"test_value"}}"#;
        write!(temp_file, "{}", test_config)?;

        // 测试加载配置
        Config::load_from_file(temp_file.path())?;
        let config = Config::get_instance().lock().unwrap();
        assert_eq!(
            config.get_setting("test_key"),
            Some(&"test_value".to_string())
        );

        Ok(())
    }
}