mod config;
mod processor;
mod utils;

use clap::{Parser, ValueEnum};
use anyhow::{Result, Context};
use std::path::PathBuf;
use std::process;

use crate::config::Config;
use crate::processor::{Operation, TextProcessor};
use crate::utils::{read_input, InputSource, safe_print};

/// 安全字符处理器 - 展示Rust所有权和生命周期特性的命令行工具
#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// 要处理的文本（如果不提供，将从标准输入或文件读取）
    #[arg(index = 1)]
    text: Option<String>,

    /// 处理操作类型
    #[arg(short, long, value_enum, default_value = "analyze")]
    operation: OperationType,

    /// 输入文件路径
    #[arg(short, long)]
    file: Option<PathBuf>,

    /// 配置文件路径
    #[arg(short, long)]
    config: Option<PathBuf>,

    /// 是否显示详细输出
    #[arg(short, long)]
    verbose: bool,
}

#[derive(Copy, Clone, PartialEq, Eq, Debug, ValueEnum)]
enum OperationType {
    /// 转换为大写
    Upper,
    /// 转换为小写
    Lower,
    /// 移除重复字符
    Dedup,
    /// 统计单词
    Count,
    /// 分析文本
    Analyze,
}

impl From<OperationType> for Operation {
    fn from(op_type: OperationType) -> Self {
        match op_type {
            OperationType::Upper => Operation::ToUpper,
            OperationType::Lower => Operation::ToLower,
            OperationType::Dedup => Operation::RemoveDuplicates,
            OperationType::Count => Operation::CountWords,
            OperationType::Analyze => Operation::Analyze,
        }
    }
}

fn main() -> Result<()> {
    // 解析命令行参数
    let args = Args::parse();
    
    // 如果提供了配置文件，加载配置
    if let Some(config_path) = &args.config {
        Config::load_from_file(config_path)
            .with_context(|| format!("加载配置文件失败: {:?}", config_path))?;
        
        if args.verbose {
            println!("已加载配置文件: {:?}", config_path);
        }
    }
    
    // 确定输入源
    let input_source = match (&args.text, &args.file) {
        (Some(text), _) => InputSource::Text(text),
        (None, Some(file)) => InputSource::File(file.to_string_lossy().to_string()),
        (None, None) => {
            if atty::is(atty::Stream::Stdin) {
                eprintln!("错误: 未提供输入。请提供文本参数、输入文件或通过管道传入内容。");
                eprintln!("使用 --help 查看帮助信息。");
                process::exit(1);
            }
            InputSource::StdIn
        }
    };
    
    // 读取输入
    let input = read_input(input_source)
        .context("读取输入失败")?;
    
    if args.verbose {
        println!("输入长度: {} 字符", input.len());
    }
    
    // 处理文本
    let processor = TextProcessor;
    let operation: Operation = args.operation.into();
    
    if args.verbose {
        println!("执行操作: {:?}", operation);
    }
    
    // 使用生命周期标注来确保引用安全
    let result = processor.process_text(&input, operation)
        .context("处理文本失败")?;
    
    // 输出结果
    safe_print(&result);
    
    Ok(())
}

// 演示生命周期和所有权的示例函数
// 这些函数不会被直接调用，但展示了Rust的所有权系统
#[allow(dead_code)]
fn ownership_examples() {
    // 示例1: 移动语义
    let s1 = String::from("hello");
    let s2 = s1;
    // println!("{}", s1); // 编译错误: s1的所有权已移动到s2
    
    // 示例2: 借用
    let s3 = String::from("world");
    let len = calculate_length(&s3);
    println!("'{}' 的长度是 {}", s3, len); // s3仍然有效
    
    // 示例3: 可变借用
    let mut s4 = String::from("hello");
    change(&mut s4);
    println!("修改后: {}", s4);
    
    // 示例4: 生命周期
    let string1 = String::from("长字符串");
    {
        let string2 = String::from("短");
        let result = longest(string1.as_str(), string2.as_str());
        println!("最长的字符串是: {}", result);
    }
}

// 借用示例
fn calculate_length(s: &String) -> usize {
    s.len()
}

// 可变借用示例
fn change(s: &mut String) {
    s.push_str(", world");
}

// 生命周期标注示例
fn longest<'a>(x: &'a str, y: &'a str) -> &'a str {
    if x.len() > y.len() { x } else { y }
}