use std::collections::{HashMap, HashSet};
use anyhow::Result;
use thiserror::Error;
use crate::config::Config;

#[derive(Debug, Error)]
pub enum ProcessorError {
    #[error("无效的输入文本")]
    InvalidInput,
    
    #[error("处理操作失败: {0}")]
    ProcessingFailed(String),
}

#[derive(Debug, Clone, PartialEq)]
pub enum Operation {
    ToUpper,
    ToLower,
    RemoveDuplicates,
    CountWords,
    Analyze,
}

impl std::str::FromStr for Operation {
    type Err = String;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s.to_lowercase().as_str() {
            "upper" => Ok(Operation::ToUpper),
            "lower" => Ok(Operation::ToLower),
            "dedup" => Ok(Operation::RemoveDuplicates),
            "count" => Ok(Operation::CountWords),
            "analyze" => Ok(Operation::Analyze),
            _ => Err(format!("未知的操作类型: {}", s)),
        }
    }
}

pub struct TextProcessor;

impl TextProcessor {
    // 处理文本的主函数，根据操作类型调用不同的处理函数
    pub fn process_text<'a>(&self, input: &'a str, operation: Operation) -> Result<String> {
        if input.is_empty() {
            return Err(ProcessorError::InvalidInput.into());
        }

        match operation {
            Operation::ToUpper => Ok(self.to_upper(input)),
            Operation::ToLower => Ok(self.to_lower(input)),
            Operation::RemoveDuplicates => Ok(self.remove_duplicates(input)),
            Operation::CountWords => Ok(self.count_words(input)),
            Operation::Analyze => Ok(self.analyze_text(input)),
        }
    }

    // 转换为大写
    pub fn to_upper(&self, text: &str) -> String {
        text.to_uppercase()
    }

    // 转换为小写
    pub fn to_lower(&self, text: &str) -> String {
        text.to_lowercase()
    }

    // 移除重复字符（保持顺序）
    pub fn remove_duplicates(&self, text: &str) -> String {
        let mut seen = HashSet::new();
        let mut result = String::with_capacity(text.len());

        for c in text.chars() {
            if seen.insert(c) {
                result.push(c);
            }
        }

        result
    }

    // 统计单词数量
    pub fn count_words(&self, text: &str) -> String {
        let config = Config::get_instance().lock().unwrap();
        let case_sensitive = config.get_setting("case_sensitive")
            .map(|s| s == "true")
            .unwrap_or(true);
        
        let min_length = config.get_setting("min_word_length")
            .and_then(|s| s.parse::<usize>().ok())
            .unwrap_or(1);
            
        let max_length = config.get_setting("max_word_length")
            .and_then(|s| s.parse::<usize>().ok())
            .unwrap_or(100);

        let mut word_counts = HashMap::new();
        
        // 分割文本为单词并计数
        for word in text.split_whitespace() {
            let word = word.trim_matches(|c: char| !c.is_alphanumeric());
            
            if word.len() < min_length || word.len() > max_length {
                continue;
            }
            
            let key = if case_sensitive {
                word.to_string()
            } else {
                word.to_lowercase()
            };
            
            *word_counts.entry(key).or_insert(0) += 1;
        }
        
        // 格式化输出结果
        let mut result = String::new();
        result.push_str("单词统计结果:\n");
        
        let mut words: Vec<(&String, &i32)> = word_counts.iter().collect();
        words.sort_by(|a, b| b.1.cmp(a.1));
        
        for (word, count) in words {
            result.push_str(&format!("{}: {}\n", word, count));
        }
        
        result
    }

    // 分析文本（字符频率、平均单词长度等）
    pub fn analyze_text(&self, text: &str) -> String {
        let char_count = text.chars().count();
        let byte_count = text.len();
        let line_count = text.lines().count();
        let word_count = text.split_whitespace().count();
        
        // 字符频率分析
        let mut char_freq = HashMap::new();
        for c in text.chars() {
            *char_freq.entry(c).or_insert(0) += 1;
        }
        
        // 计算平均单词长度
        let avg_word_length = if word_count > 0 {
            let total_word_length: usize = text.split_whitespace()
                .map(|w| w.chars().count())
                .sum();
            total_word_length as f64 / word_count as f64
        } else {
            0.0
        };
        
        // 格式化输出结果
        let mut result = String::new();
        result.push_str("文本分析结果:\n");
        result.push_str(&format!("字符数: {}\n", char_count));
        result.push_str(&format!("字节数: {}\n", byte_count));
        result.push_str(&format!("行数: {}\n", line_count));
        result.push_str(&format!("单词数: {}\n", word_count));
        result.push_str(&format!("平均单词长度: {:.2}\n", avg_word_length));
        
        result.push_str("\n最常见的字符:\n");
        let mut chars: Vec<(&char, &i32)> = char_freq.iter().collect();
        chars.sort_by(|a, b| b.1.cmp(a.1));
        
        for (i, (c, count)) in chars.iter().take(10).enumerate() {
            let display_char = if c.is_whitespace() {
                match **c {
                    ' ' => "空格".to_string(),
                    '\n' => "换行".to_string(),
                    '\t' => "制表符".to_string(),
                    _ => format!("Unicode({})", **c as u32),
                }
            } else {
                c.to_string()
            };
            
            result.push_str(&format!("{}. '{}': {} ({:.2}%)\n", 
                i + 1, 
                display_char, 
                count, 
                (**count as f64 / char_count as f64) * 100.0
            ));
        }
        
        result
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_to_upper() {
        let processor = TextProcessor;
        assert_eq!(processor.to_upper("hello world"), "HELLO WORLD");
    }

    #[test]
    fn test_to_lower() {
        let processor = TextProcessor;
        assert_eq!(processor.to_lower("HELLO WORLD"), "hello world");
    }

    #[test]
    fn test_remove_duplicates() {
        let processor = TextProcessor;
        assert_eq!(processor.remove_duplicates("hello"), "helo");
        assert_eq!(processor.remove_duplicates("aabbcc"), "abc");
    }

    #[test]
    fn test_count_words() {
        let processor = TextProcessor;
        let result = processor.count_words("hello world hello");
        assert!(result.contains("hello: 2"));
        assert!(result.contains("world: 1"));
    }

    #[test]
    fn test_analyze_text() {
        let processor = TextProcessor;
        let result = processor.analyze_text("hello world\nhello");
        assert!(result.contains("字符数: 17"));
        assert!(result.contains("行数: 2"));
        assert!(result.contains("单词数: 3"));
    }
}