use std::fs;
use std::io::{self, Read};
use std::path::Path;
use anyhow::{Result, Context};
use thiserror::Error;

#[derive(Debug, Error)]
pub enum InputError {
    #[error("文件不存在: {0}")]
    FileNotFound(String),
    
    #[error("读取文件失败: {0}")]
    ReadError(#[from] io::Error),
    
    #[error("无法处理输入: {0}")]
    ProcessingError(String),
}

/// 表示文本输入的来源
pub enum InputSource<'a> {
    /// 从文件读取
    File(String),
    /// 从标准输入读取
    StdIn,
    /// 直接使用字符串
    Text(&'a str),
}

/// 从不同来源读取文本
pub fn read_input(source: InputSource) -> Result<String> {
    match source {
        InputSource::File(path) => {
            read_from_file(&path)
                .with_context(|| format!("读取文件 '{}' 失败", path))
        },
        InputSource::StdIn => {
            read_from_stdin()
                .context("从标准输入读取失败")
        },
        InputSource::Text(text) => {
            Ok(text.to_string())
        },
    }
}

/// 从文件读取文本
pub fn read_from_file<P: AsRef<Path>>(path: P) -> Result<String> {
    let path_str = path.as_ref().to_string_lossy().to_string();
    
    if !path.as_ref().exists() {
        return Err(InputError::FileNotFound(path_str).into());
    }
    
    fs::read_to_string(path)
        .map_err(|e| InputError::ReadError(e).into())
}

/// 从标准输入读取文本
pub fn read_from_stdin() -> Result<String> {
    let mut buffer = String::new();
    io::stdin().read_to_string(&mut buffer)?;
    Ok(buffer)
}

/// 安全地打印结果
pub fn safe_print(text: &str) {
    println!("{}", text);
}

/// 检查文本是否为有效的UTF-8
pub fn is_valid_utf8(bytes: &[u8]) -> bool {
    String::from_utf8(bytes.to_vec()).is_ok()
}

/// 安全地处理文件路径
pub fn sanitize_path(path: &str) -> String {
    // 移除可能导致路径遍历的模式
    let path = path.replace("..", "");
    
    // 规范化路径分隔符
    #[cfg(target_os = "windows")]
    let path = path.replace("/", "\\");
    
    #[cfg(not(target_os = "windows"))]
    let path = path.replace("\\", "/");
    
    path
}

/// 创建一个带有生命周期的文本引用
pub struct TextRef<'a> {
    content: &'a str,
}

impl<'a> TextRef<'a> {
    /// 创建一个新的文本引用
    pub fn new(content: &'a str) -> Self {
        TextRef { content }
    }
    
    /// 获取内容
    pub fn content(&self) -> &'a str {
        self.content
    }
    
    /// 获取内容的一部分
    pub fn substring(&self, start: usize, end: usize) -> &'a str {
        let start = start.min(self.content.len());
        let end = end.min(self.content.len());
        
        if start >= end {
            return "";
        }
        
        // 确保我们在字符边界上切割
        let mut char_indices = self.content.char_indices();
        
        let start_idx = char_indices
            .find(|(i, _)| *i >= start)
            .map(|(i, _)| i)
            .unwrap_or(self.content.len());
            
        let end_idx = char_indices
            .find(|(i, _)| *i >= end)
            .map(|(i, _)| i)
            .unwrap_or(self.content.len());
            
        &self.content[start_idx..end_idx]
    }
}

/// 实现Drop trait，用于演示资源清理
impl<'a> Drop for TextRef<'a> {
    fn drop(&mut self) {
        // 在实际应用中，这里可能会执行一些清理操作
        // 这里只是为了演示Drop trait的使用
        println!("TextRef被释放，长度为: {}", self.content.len());
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use std::fs::File;
    use std::io::Write;
    use tempfile::NamedTempFile;

    #[test]
    fn test_read_from_file() -> Result<()> {
        let mut temp_file = NamedTempFile::new()?;
        write!(temp_file, "测试内容")?;
        
        let content = read_from_file(temp_file.path())?;
        assert_eq!(content, "测试内容");
        
        Ok(())
    }

    #[test]
    fn test_is_valid_utf8() {
        assert!(is_valid_utf8("你好".as_bytes()));
        assert!(!is_valid_utf8(&[0xFF, 0xFE, 0xFD]));
    }

    #[test]
    fn test_sanitize_path() {
        assert_eq!(sanitize_path("../dangerous/path"), "/dangerous/path");
    }

    #[test]
    fn test_text_ref() {
        let text = "Hello, world!";
        let text_ref = TextRef::new(text);
        
        assert_eq!(text_ref.content(), "Hello, world!");
        assert_eq!(text_ref.substring(0, 5), "Hello");
        assert_eq!(text_ref.substring(7, 12), "world");
    }
}