use std::fmt;
use std::io;
use thiserror::Error;

/// 文件处理错误类型
#[derive(Error, Debug)]
pub enum FileError {
    /// 文件未找到错误
    #[error("文件未找到: {0}")]
    NotFound(String),

    /// 权限错误
    #[error("权限不足: {0}")]
    PermissionDenied(String),

    /// 读取错误
    #[error("读取文件失败: {0}")]
    ReadError(String),

    /// 写入错误
    #[error("写入文件失败: {0}")]
    WriteError(String),

    /// 处理错误
    #[error("处理内容失败: {0}")]
    ProcessError(String),

    /// IO错误
    #[error("IO错误: {0}")]
    IoError(#[from] io::Error),

    /// 未知错误
    #[error("未知错误: {0}")]
    Unknown(String),
}

// 实现Display trait，提供更友好的错误信息
impl fmt::Display for FileError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            FileError::NotFound(path) => write!(f, "找不到文件: {}", path),
            FileError::PermissionDenied(path) => write!(f, "没有权限访问文件: {}", path),
            FileError::ReadError(msg) => write!(f, "读取文件时出错: {}", msg),
            FileError::WriteError(msg) => write!(f, "写入文件时出错: {}", msg),
            FileError::ProcessError(msg) => write!(f, "处理文件内容时出错: {}", msg),
            FileError::IoError(err) => write!(f, "IO错误: {}", err),
            FileError::Unknown(msg) => write!(f, "未知错误: {}", msg),
        }
    }
}

// 从字符串创建错误的便捷方法
impl From<&str> for FileError {
    fn from(error: &str) -> Self {
        FileError::Unknown(error.to_string())
    }
}

// 从字符串创建错误的便捷方法
impl From<String> for FileError {
    fn from(error: String) -> Self {
        FileError::Unknown(error)
    }
}