mod error;
mod file_ops;
mod traits;
mod utils;

use std::collections::HashMap;
use std::fs::File;
use std::io::Write;

use error::FileError;
use file_ops::{ProcessingPipeline, SafeFileReader, SafeFileWriter};
use traits::{ContentAnalyzer, FileProcessor};
use utils::{
    EmptyLineFilter, KeywordFilter, LineNumberProcessor, LowerCaseProcessor,
    MarkdownToHtmlTransformer, TextStatistics, TextStatisticsAnalyzer, UpperCaseProcessor,
    WordFrequencyAnalyzer,
};

/// 创建示例文件
fn create_sample_file(path: &str, content: &str) -> Result<(), FileError> {
    let mut file = File::create(path).map_err(FileError::IoError)?;
    file.write_all(content.as_bytes()).map_err(FileError::IoError)?;
    Ok(())
}

/// 展示文件处理器的使用
fn demo_file_processors() -> Result<(), FileError> {
    println!("\n=== 文件处理器演示 ===");
    
    // 创建示例文件
    let sample_path = "sample.txt";
    let sample_content = "这是第一行文本。\n这是第二行文本。\n\n这是第四行文本。";
    create_sample_file(sample_path, sample_content)?;
    println!("创建了示例文件: {}", sample_path);
    
    // 创建文件读取器
    let reader = SafeFileReader::new(sample_path.to_string());
    
    // 使用大写处理器
    let upper_processor = UpperCaseProcessor;
    let upper_result = reader.read_and_process(&upper_processor)?;
    println!("\n使用{}:\n{}", upper_processor.name(), upper_result);
    
    // 使用小写处理器
    let lower_processor = LowerCaseProcessor;
    let lower_result = reader.read_and_process(&lower_processor)?;
    println!("\n使用{}:\n{}", lower_processor.name(), lower_result);
    
    // 使用行号处理器
    let line_num_processor = LineNumberProcessor;
    let line_num_result = reader.read_and_process(&line_num_processor)?;
    println!("\n使用{}:\n{}", line_num_processor.name(), line_num_result);
    
    // 使用空行过滤器
    let empty_line_filter = EmptyLineFilter;
    let filtered_content = reader.read_and_filter(&empty_line_filter)?;
    println!("\n使用空行过滤器:\n{}", filtered_content);
    
    // 使用关键词过滤器
    let keyword_filter = KeywordFilter::new("第二".to_string(), true);
    let keyword_filtered = reader.read_and_filter(&keyword_filter)?;
    println!("\n使用关键词过滤器 (关键词: '第二'):\n{}", keyword_filtered);
    
    Ok(())
}

/// 展示处理管道的使用
fn demo_processing_pipeline() -> Result<(), FileError> {
    println!("\n=== 处理管道演示 ===");
    
    // 创建示例文件
    let sample_path = "pipeline_sample.txt";
    let sample_content = "第一行\n第二行\n第三行\n第四行";
    create_sample_file(sample_path, sample_content)?;
    println!("创建了示例文件: {}", sample_path);
    
    // 创建文件读取器
    let reader = SafeFileReader::new(sample_path.to_string());
    let content = reader.read()?;
    
    // 创建处理管道
    let mut pipeline = ProcessingPipeline::new();
    
    // 添加处理器
    pipeline
        .add_processor(&LineNumberProcessor)
        .add_processor(&UpperCaseProcessor);
    
    // 执行管道
    let result = pipeline.process(&content)?;
    println!("\n处理管道结果 (行号 -> 大写):\n{}", result);
    
    Ok(())
}

/// 展示内容分析器的使用
fn demo_content_analyzers() -> Result<(), FileError> {
    println!("\n=== 内容分析器演示 ===");
    
    // 创建示例文件
    let sample_path = "analyzer_sample.txt";
    let sample_content = "这是一个示例文本文件。\n它包含多行文本。\n\n这些文本将被分析。\n我们可以统计单词频率和其他指标。";
    create_sample_file(sample_path, sample_content)?;
    println!("创建了示例文件: {}", sample_path);
    
    // 创建文件读取器
    let reader = SafeFileReader::new(sample_path.to_string());
    
    // 使用文本统计分析器
    let stats_analyzer = TextStatisticsAnalyzer;
    let stats: TextStatistics = reader.read_and_analyze(&stats_analyzer)?;
    
    println!("\n文本统计结果:");
    println!("总行数: {}", stats.line_count);
    println!("总字数: {}", stats.word_count);
    println!("总字符数: {}", stats.char_count);
    println!("非空行数: {}", stats.non_empty_lines);
    
    // 使用单词频率分析器
    let word_analyzer = WordFrequencyAnalyzer;
    let word_counts: HashMap<String, usize> = reader.read_and_analyze(&word_analyzer)?;
    
    println!("\n单词频率统计 (前5个):");
    let mut words: Vec<(&String, &usize)> = word_counts.iter().collect();
    words.sort_by(|a, b| b.1.cmp(a.1));
    
    for (i, (word, count)) in words.iter().take(5).enumerate() {
        println!("{}. {} - 出现{}次", i + 1, word, count);
    }
    
    Ok(())
}

/// 展示内容转换器的使用
fn demo_content_transformer() -> Result<(), FileError> {
    println!("\n=== 内容转换器演示 ===");
    
    // 创建示例Markdown文件
    let md_path = "sample.md";
    let md_content = "# 示例标题\n\n## 小标题\n\n这是一段普通文本。\n\n### 小小标题\n\n这是另一段文本。";
    create_sample_file(md_path, md_content)?;
    println!("创建了示例Markdown文件: {}", md_path);
    
    // 创建文件读取器和写入器
    let reader = SafeFileReader::new(md_path.to_string());
    let writer = SafeFileWriter::new("sample.html".to_string());
    
    // 读取Markdown内容
    let content = reader.read()?;
    
    // 使用Markdown到HTML转换器
    let transformer = MarkdownToHtmlTransformer;
    writer.transform_and_write(&content, &transformer)?;
    
    println!("\nMarkdown已转换为HTML并保存到 sample.html");
    
    Ok(())
}

/// 自定义处理器示例
struct ReverseLineProcessor;

impl FileProcessor for ReverseLineProcessor {
    type Error = String;
    
    fn process(&self, content: &str) -> Result<String, Self::Error> {
        let reversed = content
            .lines()
            .map(|line| line.chars().rev().collect::<String>())
            .collect::<Vec<String>>()
            .join("\n");
            
        Ok(reversed)
    }
    
    fn name(&self) -> String {
        String::from("行反转处理器")
    }
    
    fn description(&self) -> String {
        String::from("将每行文本反转")
    }
}

fn main() -> Result<(), FileError> {
    println!("=== 文件处理工具库演示 ===");
    
    // 演示文件处理器
    demo_file_processors()?;
    
    // 演示处理管道
    demo_processing_pipeline()?;
    
    // 演示内容分析器
    demo_content_analyzers()?;
    
    // 演示内容转换器
    demo_content_transformer()?;
    
    // 使用自定义处理器
    println!("\n=== 自定义处理器演示 ===");
    let sample_path = "custom_sample.txt";
    let sample_content = "这是第一行\n这是第二行\n这是第三行";
    create_sample_file(sample_path, sample_content)?;
    
    let reader = SafeFileReader::new(sample_path.to_string());
    let reverse_processor = ReverseLineProcessor;
    let result = reader.read_and_process(&reverse_processor)?;
    
    println!("\n使用{}:\n{}", reverse_processor.name(), result);
    
    println!("\n所有演示完成！");
    
    Ok(())
}