use std::fmt::Debug;

/// 文件处理器trait
/// 
/// 定义了处理文件内容的通用接口
pub trait FileProcessor {
    /// 处理器可能产生的错误类型
    type Error: Debug;
    
    /// 处理文件内容的方法
    /// 
    /// # 参数
    /// * `content` - 要处理的文件内容
    /// 
    /// # 返回
    /// * `Result<String, Self::Error>` - 处理后的内容或错误
    fn process(&self, content: &str) -> Result<String, Self::Error>;
    
    /// 处理器名称（默认实现）
    fn name(&self) -> String {
        String::from("默认处理器")
    }
    
    /// 处理器描述（默认实现）
    fn description(&self) -> String {
        String::from("处理文件内容")
    }
}

/// 文件过滤器trait
/// 
/// 用于过滤文件内容的行
pub trait LineFilter {
    /// 判断一行是否应该被保留
    /// 
    /// # 参数
    /// * `line` - 要判断的行
    /// 
    /// # 返回
    /// * `bool` - true表示保留，false表示过滤掉
    fn should_keep(&self, line: &str) -> bool;
}

/// 文件转换器trait
/// 
/// 用于转换文件内容
pub trait ContentTransformer {
    /// 转换器可能产生的错误类型
    type Error: Debug;
    
    /// 转换文件内容
    /// 
    /// # 参数
    /// * `content` - 要转换的内容
    /// 
    /// # 返回
    /// * `Result<String, Self::Error>` - 转换后的内容或错误
    fn transform(&self, content: &str) -> Result<String, Self::Error>;
}

/// 文件分析器trait
/// 
/// 用于分析文件内容并生成统计信息
pub trait ContentAnalyzer<T> {
    /// 分析器可能产生的错误类型
    type Error: Debug;
    
    /// 分析文件内容
    /// 
    /// # 参数
    /// * `content` - 要分析的内容
    /// 
    /// # 返回
    /// * `Result<T, Self::Error>` - 分析结果或错误
    fn analyze(&self, content: &str) -> Result<T, Self::Error>;
}