use std::collections::HashMap;
use std::path::Path;

use crate::traits::{ContentAnalyzer, ContentTransformer, FileProcessor, LineFilter};

/// 大写转换处理器
pub struct UpperCaseProcessor;

impl FileProcessor for UpperCaseProcessor {
    type Error = String;
    
    fn process(&self, content: &str) -> Result<String, Self::Error> {
        Ok(content.to_uppercase())
    }
    
    fn name(&self) -> String {
        String::from("大写转换处理器")
    }
    
    fn description(&self) -> String {
        String::from("将文本内容转换为大写")
    }
}

/// 小写转换处理器
pub struct LowerCaseProcessor;

impl FileProcessor for LowerCaseProcessor {
    type Error = String;
    
    fn process(&self, content: &str) -> Result<String, Self::Error> {
        Ok(content.to_lowercase())
    }
    
    fn name(&self) -> String {
        String::from("小写转换处理器")
    }
    
    fn description(&self) -> String {
        String::from("将文本内容转换为小写")
    }
}

/// 行号添加处理器
pub struct LineNumberProcessor;

impl FileProcessor for LineNumberProcessor {
    type Error = String;
    
    fn process(&self, content: &str) -> Result<String, Self::Error> {
        let numbered = content
            .lines()
            .enumerate()
            .map(|(i, line)| format!("{}: {}", i + 1, line))
            .collect::<Vec<String>>()
            .join("\n");
            
        Ok(numbered)
    }
    
    fn name(&self) -> String {
        String::from("行号添加处理器")
    }
    
    fn description(&self) -> String {
        String::from("为每行添加行号")
    }
}

/// 空行过滤器
pub struct EmptyLineFilter;

impl LineFilter for EmptyLineFilter {
    fn should_keep(&self, line: &str) -> bool {
        !line.trim().is_empty()
    }
}

/// 关键词过滤器
pub struct KeywordFilter {
    keyword: String,
    case_sensitive: bool,
}

impl KeywordFilter {
    /// 创建新的关键词过滤器
    /// 
    /// # 参数
    /// * `keyword` - 要过滤的关键词
    /// * `case_sensitive` - 是否区分大小写
    pub fn new(keyword: String, case_sensitive: bool) -> Self {
        KeywordFilter {
            keyword,
            case_sensitive,
        }
    }
}

impl LineFilter for KeywordFilter {
    fn should_keep(&self, line: &str) -> bool {
        if self.case_sensitive {
            line.contains(&self.keyword)
        } else {
            line.to_lowercase().contains(&self.keyword.to_lowercase())
        }
    }
}

/// Markdown转HTML转换器
pub struct MarkdownToHtmlTransformer;

impl ContentTransformer for MarkdownToHtmlTransformer {
    type Error = String;
    
    fn transform(&self, content: &str) -> Result<String, Self::Error> {
        // 这里只是一个简单的实现，真实场景可能需要使用专门的Markdown解析库
        let mut html = String::from("<html><body>\n");
        
        for line in content.lines() {
            if line.starts_with("# ") {
                html.push_str(&format!("<h1>{}</h1>\n", &line[2..]));
            } else if line.starts_with("## ") {
                html.push_str(&format!("<h2>{}</h2>\n", &line[3..]));
            } else if line.starts_with("### ") {
                html.push_str(&format!("<h3>{}</h3>\n", &line[4..]));
            } else if line.trim().is_empty() {
                html.push_str("<br>\n");
            } else {
                html.push_str(&format!("<p>{}</p>\n", line));
            }
        }
        
        html.push_str("</body></html>");
        Ok(html)
    }
}

/// 文本统计分析器
pub struct TextStatisticsAnalyzer;

/// 文本统计结果
#[derive(Debug)]
pub struct TextStatistics {
    /// 总行数
    pub line_count: usize,
    /// 总字数
    pub word_count: usize,
    /// 总字符数
    pub char_count: usize,
    /// 非空行数
    pub non_empty_lines: usize,
}

impl ContentAnalyzer<TextStatistics> for TextStatisticsAnalyzer {
    type Error = String;
    
    fn analyze(&self, content: &str) -> Result<TextStatistics, Self::Error> {
        let lines: Vec<&str> = content.lines().collect();
        let line_count = lines.len();
        
        let char_count = content.chars().count();
        let word_count = content
            .split_whitespace()
            .count();
            
        let non_empty_lines = lines
            .iter()
            .filter(|line| !line.trim().is_empty())
            .count();
            
        Ok(TextStatistics {
            line_count,
            word_count,
            char_count,
            non_empty_lines,
        })
    }
}

/// 单词频率分析器
pub struct WordFrequencyAnalyzer;

impl ContentAnalyzer<HashMap<String, usize>> for WordFrequencyAnalyzer {
    type Error = String;
    
    fn analyze(&self, content: &str) -> Result<HashMap<String, usize>, Self::Error> {
        let mut word_counts = HashMap::new();
        
        for word in content.split_whitespace() {
            // 移除标点符号并转为小写
            let clean_word = word
                .chars()
                .filter(|c| c.is_alphanumeric())
                .collect::<String>()
                .to_lowercase();
                
            if !clean_word.is_empty() {
                *word_counts.entry(clean_word).or_insert(0) += 1;
            }
        }
        
        Ok(word_counts)
    }
}

/// 文件扩展名检查器
pub struct FileExtensionChecker {
    allowed_extensions: Vec<String>,
}

impl FileExtensionChecker {
    /// 创建新的文件扩展名检查器
    /// 
    /// # 参数
    /// * `extensions` - 允许的扩展名列表
    pub fn new(extensions: Vec<String>) -> Self {
        FileExtensionChecker {
            allowed_extensions: extensions,
        }
    }
    
    /// 检查文件扩展名是否允许
    /// 
    /// # 参数
    /// * `file_path` - 文件路径
    /// 
    /// # 返回
    /// * `bool` - 是否允许
    pub fn is_allowed(&self, file_path: &str) -> bool {
        let path = Path::new(file_path);
        
        if let Some(extension) = path.extension() {
            if let Some(ext_str) = extension.to_str() {
                return self.allowed_extensions.iter().any(|allowed| allowed == ext_str);
            }
        }
        
        false
    }
}