//! 图书相关的数据模型

use std::fmt;

/// 表示图书馆中的一本书
#[derive(Debug, Clone, PartialEq)]
pub struct Book {
    title: String,
    author: String,
    isbn: String,
    available: bool,
}

impl Book {
    /// 创建一本新书
    ///
    /// # 示例
    /// ```
    /// use library::Book;
    /// let book = Book::new("Rust编程艺术", "张三", "978-1234567890");
    /// assert_eq!(book.title(), "Rust编程艺术");
    /// assert!(book.is_available());
    /// ```
    pub fn new(title: &str, author: &str, isbn: &str) -> Self {
        Self {
            title: title.to_string(),
            author: author.to_string(),
            isbn: isbn.to_string(),
            available: true,
        }
    }
    
    /// 获取书名
    pub fn title(&self) -> &str {
        &self.title
    }
    
    /// 获取作者
    pub fn author(&self) -> &str {
        &self.author
    }
    
    /// 获取ISBN
    pub fn isbn(&self) -> &str {
        &self.isbn
    }
    
    /// 检查书是否可借
    pub fn is_available(&self) -> bool {
        self.available
    }
    
    /// 标记为已借出
    pub(crate) fn mark_borrowed(&mut self) {
        self.available = false;
    }
    
    /// 标记为已归还
    pub(crate) fn mark_returned(&mut self) {
        self.available = true;
    }
}

impl fmt::Display for Book {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{} by {} (ISBN: {})", self.title, self.author, self.isbn)
    }
}