//! 用户相关的数据模型

use std::fmt;

/// 用户ID类型
pub type UserId = u32;

/// 表示图书馆的一名用户
#[derive(Debug, Clone, PartialEq)]
pub struct User {
    id: UserId,
    name: String,
    email: String,
    active: bool,
    borrowed_books: Vec<String>, // 存储已借阅书籍的ISBN
}

impl User {
    /// 创建一个新用户
    ///
    /// # 示例
    /// ```
    /// use library::User;
    /// let user = User::new(1, "李四", "lisi@example.com");
    /// assert_eq!(user.name(), "李四");
    /// assert!(user.is_active());
    /// ```
    pub fn new(id: UserId, name: &str, email: &str) -> Self {
        Self {
            id,
            name: name.to_string(),
            email: email.to_string(),
            active: true,
            borrowed_books: Vec::new(),
        }
    }
    
    /// 获取用户ID
    pub fn id(&self) -> UserId {
        self.id
    }
    
    /// 获取用户名
    pub fn name(&self) -> &str {
        &self.name
    }
    
    /// 获取用户邮箱
    pub fn email(&self) -> &str {
        &self.email
    }
    
    /// 检查用户是否激活
    pub fn is_active(&self) -> bool {
        self.active
    }
    
    /// 获取用户已借阅的书籍ISBN列表
    pub fn borrowed_books(&self) -> &[String] {
        &self.borrowed_books
    }
    
    /// 记录用户借阅了一本书
    pub(crate) fn borrow_book(&mut self, isbn: &str) {
        self.borrowed_books.push(isbn.to_string());
    }
    
    /// 记录用户归还了一本书
    pub(crate) fn return_book(&mut self, isbn: &str) -> bool {
        if let Some(pos) = self.borrowed_books.iter().position(|b| b == isbn) {
            self.borrowed_books.remove(pos);
            true
        } else {
            false
        }
    }
    
    /// 停用用户账户
    pub(crate) fn deactivate(&mut self) {
        self.active = false;
    }
    
    /// 重新激活用户账户
    pub(crate) fn activate(&mut self) {
        self.active = true;
    }
}

impl fmt::Display for User {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{} (ID: {}, Email: {})", self.name, self.id, self.email)
    }
}