//! 图书目录管理服务

use std::collections::HashMap;
use crate::models::Book;
use crate::utils::validation;

/// 图书目录错误类型
#[derive(Debug)]
pub enum CatalogError {
    /// ISBN已存在
    DuplicateIsbn(String),
    /// ISBN不存在
    IsbnNotFound(String),
    /// ISBN格式无效
    InvalidIsbn(String),
}

/// 图书目录服务
pub struct Catalog {
    books: HashMap<String, Book>,
}

impl Catalog {
    /// 创建一个新的目录服务
    pub fn new() -> Self {
        Self {
            books: HashMap::new(),
        }
    }
    
    /// 添加一本新书到目录
    ///
    /// # 错误
    /// 如果ISBN已存在或无效，返回错误
    ///
    /// # 示例
    /// ```
    /// use library::{Book, Catalog};
    /// 
    /// let mut catalog = Catalog::new();
    /// let book = Book::new("Rust编程", "张三", "978-1234567890");
    /// 
    /// // 首次添加成功
    /// assert!(catalog.add_book(book.clone()).is_ok());
    /// 
    /// // 重复添加失败
    /// assert!(catalog.add_book(book).is_err());
    /// ```
    pub fn add_book(&mut self, book: Book) -> Result<(), CatalogError> {
        let isbn = book.isbn();
        
        // 验证ISBN格式
        if !validation::is_valid_isbn(isbn) {
            return Err(CatalogError::InvalidIsbn(isbn.to_string()));
        }
        
        // 检查ISBN是否已存在
        if self.books.contains_key(isbn) {
            return Err(CatalogError::DuplicateIsbn(isbn.to_string()));
        }
        
        // 添加图书
        self.books.insert(isbn.to_string(), book);
        Ok(())
    }
    
    /// 根据ISBN查找图书
    pub fn find_by_isbn(&self, isbn: &str) -> Option<&Book> {
        self.books.get(isbn)
    }
    
    /// 根据ISBN查找可变图书引用
    pub(crate) fn find_by_isbn_mut(&mut self, isbn: &str) -> Option<&mut Book> {
        self.books.get_mut(isbn)
    }
    
    /// 根据标题查找图书
    pub fn find_by_title(&self, title: &str) -> Vec<&Book> {
        self.books.values()
            .filter(|book| book.title().contains(title))
            .collect()
    }
    
    /// 根据作者查找图书
    pub fn find_by_author(&self, author: &str) -> Vec<&Book> {
        self.books.values()
            .filter(|book| book.author().contains(author))
            .collect()
    }
    
    /// 获取所有图书
    pub fn all_books(&self) -> Vec<&Book> {
        self.books.values().collect()
    }
    
    /// 获取可用图书数量
    pub fn available_count(&self) -> usize {
        self.books.values()
            .filter(|book| book.is_available())
            .count()
    }
    
    /// 获取总图书数量
    pub fn total_count(&self) -> usize {
        self.books.len()
    }
}

impl Default for Catalog {
    fn default() -> Self {
        Self::new()
    }
}