//! 图书借阅管理服务

use std::collections::HashMap;
use crate::models::User;
use crate::models::user::UserId;
use crate::services::catalog::Catalog;

/// 借阅记录
#[derive(Debug)]
pub struct LoanRecord {
    user_id: UserId,
    isbn: String,
    loan_date: chrono::DateTime<chrono::Utc>,
    due_date: chrono::DateTime<chrono::Utc>,
}

/// 借阅错误类型
#[derive(Debug)]
pub enum LendingError {
    /// 用户不存在
    UserNotFound(UserId),
    /// 图书不存在
    BookNotFound(String),
    /// 图书不可用（已借出）
    BookUnavailable(String),
    /// 用户账户未激活
    UserInactive(UserId),
    /// 用户已达到最大借阅限制
    BorrowLimitReached(UserId),
    /// 借阅记录不存在
    LoanNotFound,
}

/// 借阅服务
pub struct LendingService {
    users: HashMap<UserId, User>,
    active_loans: Vec<LoanRecord>,
    max_loans_per_user: usize,
    loan_duration_days: i64,
}

impl LendingService {
    /// 创建一个新的借阅服务
    pub fn new() -> Self {
        Self {
            users: HashMap::new(),
            active_loans: Vec::new(),
            max_loans_per_user: 5,
            loan_duration_days: 14,
        }
    }
    
    /// 设置每位用户的最大借阅数量
    pub fn set_max_loans_per_user(&mut self, max: usize) {
        self.max_loans_per_user = max;
    }
    
    /// 设置借阅期限（天数）
    pub fn set_loan_duration(&mut self, days: i64) {
        self.loan_duration_days = days;
    }
    
    /// 添加用户
    pub fn add_user(&mut self, user: User) {
        self.users.insert(user.id(), user);
    }
    
    /// 查找用户
    pub fn find_user(&self, user_id: UserId) -> Option<&User> {
        self.users.get(&user_id)
    }
    
    /// 查找可变用户引用
    fn find_user_mut(&mut self, user_id: UserId) -> Option<&mut User> {
        self.users.get_mut(&user_id)
    }
    
    /// 借阅图书
    pub fn borrow_book(
        &mut self, 
        user_id: UserId, 
        isbn: &str, 
        catalog: &mut Catalog
    ) -> Result<(), LendingError> {
        // 先获取最大借阅数量
        let max_loans = self.max_loans_per_user;
        
        // 检查用户是否存在
        let user = self.find_user_mut(user_id)
            .ok_or(LendingError::UserNotFound(user_id))?;
        
        // 检查用户是否激活
        if !user.is_active() {
            return Err(LendingError::UserInactive(user_id));
        }
        
        // 检查用户是否达到借阅上限
        if user.borrowed_books().len() >= max_loans {
            return Err(LendingError::BorrowLimitReached(user_id));
        }
        
        // 检查图书是否存在并可借
        let book = catalog.find_by_isbn_mut(isbn)
            .ok_or_else(|| LendingError::BookNotFound(isbn.to_string()))?;
        
        if !book.is_available() {
            return Err(LendingError::BookUnavailable(isbn.to_string()));
        }
        
        // 更新图书状态
        book.mark_borrowed();
        
        // 更新用户借阅记录
        user.borrow_book(isbn);
        
        // 创建借阅记录
        let now = chrono::Utc::now();
        let due_date = now + chrono::Duration::days(self.loan_duration_days);
        
        self.active_loans.push(LoanRecord {
            user_id,
            isbn: isbn.to_string(),
            loan_date: now,
            due_date,
        });
        
        Ok(())
    }
    
    /// 归还图书
    pub fn return_book(
        &mut self, 
        user_id: UserId, 
        isbn: &str, 
        catalog: &mut Catalog
    ) -> Result<(), LendingError> {
        // 检查用户是否存在
        let user = self.find_user_mut(user_id)
            .ok_or(LendingError::UserNotFound(user_id))?;
        
        // 检查图书是否存在
        let book = catalog.find_by_isbn_mut(isbn)
            .ok_or_else(|| LendingError::BookNotFound(isbn.to_string()))?;
        
        // 更新用户借阅记录
        if !user.return_book(isbn) {
            return Err(LendingError::LoanNotFound);
        }
        
        // 更新图书状态
        book.mark_returned();
        
        // 移除借阅记录
        let loan_index = self.active_loans.iter()
            .position(|loan| loan.user_id == user_id && loan.isbn == isbn)
            .ok_or(LendingError::LoanNotFound)?;
        
        self.active_loans.remove(loan_index);
        
        Ok(())
    }
    
    /// 获取用户的所有借阅
    pub fn get_user_loans(&self, user_id: UserId) -> Vec<&LoanRecord> {
        self.active_loans.iter()
            .filter(|loan| loan.user_id == user_id)
            .collect()
    }
    
    /// 获取所有逾期借阅
    pub fn get_overdue_loans(&self) -> Vec<&LoanRecord> {
        let now = chrono::Utc::now();
        self.active_loans.iter()
            .filter(|loan| loan.due_date < now)
            .collect()
    }
}

impl Default for LendingService {
    fn default() -> Self {
        Self::new()
    }
}