//! 数据验证工具

/// 检查ISBN是否有效
/// 
/// 简化的ISBN-13验证，仅检查格式和长度
/// 
/// # 示例
/// ```
/// use library::utils::validation::is_valid_isbn;
/// 
/// assert!(is_valid_isbn("978-1234567890"));
/// assert!(!is_valid_isbn("invalid-isbn"));
/// ```
pub fn is_valid_isbn(isbn: &str) -> bool {
    // 移除所有连字符
    let cleaned = isbn.replace('-', "");
    
    // 检查长度（ISBN-13应为13位数字）
    if cleaned.len() != 13 {
        return false;
    }
    
    // 检查是否全为数字
    cleaned.chars().all(|c| c.is_ascii_digit())
}

/// 检查电子邮件格式是否有效
/// 
/// 简化的电子邮件验证，仅检查基本格式
pub fn is_valid_email(email: &str) -> bool {
    // 检查是否包含@符号
    if !email.contains('@') {
        return false;
    }
    
    // 检查@符号前后是否有内容
    let parts: Vec<&str> = email.split('@').collect();
    if parts.len() != 2 || parts[0].is_empty() || parts[1].is_empty() {
        return false;
    }
    
    // 检查域名部分是否包含至少一个点
    if !parts[1].contains('.') {
        return false;
    }
    
    true
}

#[cfg(test)]
mod tests {
    use super::*;
    
    #[test]
    fn test_valid_isbn() {
        assert!(is_valid_isbn("9781234567890"));
        assert!(is_valid_isbn("978-1234567890"));
    }
    
    #[test]
    fn test_invalid_isbn() {
        assert!(!is_valid_isbn("123"));
        assert!(!is_valid_isbn("abcdefghijklm"));
        assert!(!is_valid_isbn("978-12345"));
    }
    
    #[test]
    fn test_valid_email() {
        assert!(is_valid_email("user@example.com"));
        assert!(is_valid_email("name.surname@domain.co.uk"));
    }
    
    #[test]
    fn test_invalid_email() {
        assert!(!is_valid_email("invalid"));
        assert!(!is_valid_email("@domain.com"));
        assert!(!is_valid_email("user@"));
        assert!(!is_valid_email("user@domain"));
    }
}